@@ -1,23 +1,23 @@
 //! Algorithms for parsing strings to floats.
 
 // Hide implementation details.
-mod exponent;
+#[macro_use]
+mod format;
 
 cfg_if! {
 if #[cfg(feature = "correct")] {
-pub(crate) mod bhcomp;
-pub(crate) mod bigcomp;
+mod bhcomp;
+mod bigcomp;
 mod alias;
-mod bigint;
+mod bignum;
 mod cached;
 mod cached_float80;
+mod cached_float160;
+mod errors;
 mod large_powers;
 mod math;
 mod small_powers;
 
-#[cfg(has_i128)]
-mod cached_float160;
-
 #[cfg(limb_width_32)]
 mod large_powers_32;
 
@@ -32,7 +32,6 @@ mod small_powers_64;
 
 }}  // cfg_if
 
-
 // Export algorithms.
 #[cfg(feature = "correct")]
 pub(crate) mod correct;
