@@ -1,6 +1,6 @@
 //! Defines rounding schemes for floating-point numbers.
 
-use util::*;
+use crate::util::*;
 use super::float::ExtendedFloat;
 use super::mantissa::Mantissa;
 use super::shift::*;
@@ -10,10 +10,10 @@ use super::shift::*;
 
 // NEAREST ROUNDING
 
-/// Shift right N-bytes and round to the nearest.
-///
-/// Return if we are above halfway and if we are halfway.
-#[inline]
+// Shift right N-bytes and round to the nearest.
+//
+// Return if we are above halfway and if we are halfway.
+perftools_inline!{
 pub(crate) fn round_nearest<M>(fp: &mut ExtendedFloat<M>, shift: i32)
     -> (bool, bool)
     where M: Mantissa
@@ -35,10 +35,10 @@ pub(crate) fn round_nearest<M>(fp: &mut ExtendedFloat<M>, shift: i32)
     overflowing_shr(fp, shift);
 
     (is_above, is_halfway)
-}
+}}
 
-/// Tie rounded floating point to event.
-#[inline]
+// Tie rounded floating point to event.
+perftools_inline!{
 pub(crate) fn tie_even<M>(fp: &mut ExtendedFloat<M>, is_above: bool, is_halfway: bool)
     where M: Mantissa
 {
@@ -51,23 +51,23 @@ pub(crate) fn tie_even<M>(fp: &mut ExtendedFloat<M>, is_above: bool, is_halfway:
     if is_above || (is_odd && is_halfway) {
         fp.mant += M::ONE;
     }
-}
+}}
 
-/// Shift right N-bytes and round nearest, tie-to-even.
-///
-/// Floating-point arithmetic uses round to nearest, ties to even,
-/// which rounds to the nearest value, if the value is halfway in between,
-/// round to an even value.
-#[inline]
+// Shift right N-bytes and round nearest, tie-to-even.
+//
+// Floating-point arithmetic uses round to nearest, ties to even,
+// which rounds to the nearest value, if the value is halfway in between,
+// round to an even value.
+perftools_inline!{
 pub(crate) fn round_nearest_tie_even<M>(fp: &mut ExtendedFloat<M>, shift: i32)
     where M: Mantissa
 {
     let (is_above, is_halfway) = round_nearest(fp, shift);
     tie_even(fp, is_above, is_halfway);
-}
+}}
 
-/// Tie rounded floating point away from zero.
-#[inline]
+// Tie rounded floating point away from zero.
+perftools_inline!{
 pub(crate) fn tie_away_zero<M>(fp: &mut ExtendedFloat<M>, is_above: bool, is_halfway: bool)
     where M: Mantissa
 {
@@ -78,27 +78,27 @@ pub(crate) fn tie_away_zero<M>(fp: &mut ExtendedFloat<M>, is_above: bool, is_hal
     if is_above || is_halfway {
         fp.mant += M::ONE;
     }
-}
+}}
 
-/// Shift right N-bytes and round nearest, tie-away-zero.
-///
-/// Floating-point arithmetic defines round to nearest, ties away from zero,
-/// which rounds to the nearest value, if the value is halfway in between,
-/// ties away from zero.
-#[inline]
+// Shift right N-bytes and round nearest, tie-away-zero.
+//
+// Floating-point arithmetic defines round to nearest, ties away from zero,
+// which rounds to the nearest value, if the value is halfway in between,
+// ties away from zero.
+perftools_inline!{
 pub(crate) fn round_nearest_tie_away_zero<M>(fp: &mut ExtendedFloat<M>, shift: i32)
     where M: Mantissa
 {
     let (is_above, is_halfway) = round_nearest(fp, shift);
     tie_away_zero(fp, is_above, is_halfway);
-}
+}}
 
 // DIRECTED ROUNDING
 
-/// Shift right N-bytes and round towards a direction.
-///
-/// Return if we have any truncated bytes.
-#[inline]
+// Shift right N-bytes and round towards a direction.
+//
+// Return if we have any truncated bytes.
+perftools_inline!{
 pub(crate) fn round_toward<M>(fp: &mut ExtendedFloat<M>, shift: i32)
     -> bool
     where M: Mantissa
@@ -110,23 +110,23 @@ pub(crate) fn round_toward<M>(fp: &mut ExtendedFloat<M>, shift: i32)
     overflowing_shr(fp, shift);
 
     truncated_bits != M::ZERO
-}
+}}
 
-/// Round up.
-#[inline]
+// Round up.
+perftools_inline!{
 pub(crate) fn upward<M>(fp: &mut ExtendedFloat<M>, is_truncated: bool)
     where M: Mantissa
 {
     if is_truncated {
         fp.mant += M::ONE;
     }
-}
+}}
 
-/// Shift right N-bytes and round toward infinity.
-///
-/// Floating-point arithmetic defines round toward infinity, which rounds
-/// towards positive infinity.
-#[inline]
+// Shift right N-bytes and round toward infinity.
+//
+// Floating-point arithmetic defines round toward infinity, which rounds
+// towards positive infinity.
+perftools_inline!{
 pub(crate) fn round_upward<M>(fp: &mut ExtendedFloat<M>, shift: i32)
     where M: Mantissa
 {
@@ -134,19 +134,19 @@ pub(crate) fn round_upward<M>(fp: &mut ExtendedFloat<M>, shift: i32)
     // round-up.
     let is_truncated = round_toward(fp, shift);
     upward(fp, is_truncated);
-}
+}}
 
-/// Round down.
-#[inline]
+// Round down.
+perftools_inline!{
 pub(crate) fn downard<M>(_: &mut ExtendedFloat<M>, _: bool)
     where M: Mantissa
-{}
+{}}
 
-/// Shift right N-bytes and round toward zero.
-///
-/// Floating-point arithmetic defines round toward zero, which rounds
-/// towards positive zero.
-#[inline]
+// Shift right N-bytes and round toward zero.
+//
+// Floating-point arithmetic defines round toward zero, which rounds
+// towards positive zero.
+perftools_inline!{
 pub(crate) fn round_downward<M>(fp: &mut ExtendedFloat<M>, shift: i32)
     where M: Mantissa
 {
@@ -154,7 +154,7 @@ pub(crate) fn round_downward<M>(fp: &mut ExtendedFloat<M>, shift: i32)
     // No rounding schemes, so we just ignore everything else.
     let is_truncated = round_toward(fp, shift);
     downard(fp, is_truncated);
-}
+}}
 
 // NATIVE FLOAT
 // ------------
@@ -179,12 +179,8 @@ macro_rules! float_rounding_f32 {
     )*)
 }
 
-#[cfg(has_i128)]
 float_rounding_f32! { u64 u128 }
 
-#[cfg(not(has_i128))]
-float_rounding_f32! { u64 }
-
 // Literals don't work for generic types, we need to use this as a hack.
 macro_rules! float_rounding_f64 {
     ($($t:tt)*) => ($(
@@ -195,20 +191,16 @@ macro_rules! float_rounding_f64 {
     )*)
 }
 
-#[cfg(has_i128)]
 float_rounding_f64! { u64 u128 }
 
-#[cfg(not(has_i128))]
-float_rounding_f64! { u64 }
-
 // ROUND TO FLOAT
 
-/// Shift the ExtendedFloat fraction to the fraction bits in a native float.
-///
-/// Floating-point arithmetic uses round to nearest, ties to even,
-/// which rounds to the nearest value, if the value is halfway in between,
-/// round to an even value.
-#[inline]
+// Shift the ExtendedFloat fraction to the fraction bits in a native float.
+//
+// Floating-point arithmetic uses round to nearest, ties to even,
+// which rounds to the nearest value, if the value is halfway in between,
+// round to an even value.
+perftools_inline!{
 pub(crate) fn round_to_float<T, M, Cb>(fp: &mut ExtendedFloat<M>, cb: Cb)
     where T: FloatRounding<M>,
           M: Mantissa,
@@ -241,14 +233,14 @@ pub(crate) fn round_to_float<T, M, Cb>(fp: &mut ExtendedFloat<M>, cb: Cb)
         // Roundup carried over to 1 past the hidden bit.
         shr(fp, 1);
     }
-}
+}}
 
 // AVOID OVERFLOW/UNDERFLOW
 
-/// Avoid overflow for large values, shift left as needed.
-///
-/// Shift until a 1-bit is in the hidden bit, if the mantissa is not 0.
-#[inline]
+// Avoid overflow for large values, shift left as needed.
+//
+// Shift until a 1-bit is in the hidden bit, if the mantissa is not 0.
+perftools_inline!{
 pub(crate) fn avoid_overflow<T, M>(fp: &mut ExtendedFloat<M>)
     where T: FloatRounding<M>,
           M: Mantissa
@@ -272,12 +264,12 @@ pub(crate) fn avoid_overflow<T, M>(fp: &mut ExtendedFloat<M>)
             }
         }
     }
-}
+}}
 
 // ROUND TO NATIVE
 
-/// Round an extended-precision float to a native float representation.
-#[inline]
+// Round an extended-precision float to a native float representation.
+perftools_inline!{
 pub(crate) fn round_to_native<T, M, Cb>(fp: &mut ExtendedFloat<M>, cb: Cb)
     where T: FloatRounding<M>,
           M: Mantissa,
@@ -291,10 +283,10 @@ pub(crate) fn round_to_native<T, M, Cb>(fp: &mut ExtendedFloat<M>, cb: Cb)
     // and avoid overflow/underflow.
     round_to_float::<T, M, _>(fp, cb);
     avoid_overflow::<T, M>(fp);
-}
+}}
 
-/// Get the rounding scheme to determine if we should go up or down.
-#[inline]
+// Get the rounding scheme to determine if we should go up or down.
+perftools_inline!{
 #[allow(unused_variables)]
 pub(crate) fn internal_rounding(kind: RoundingKind, sign: Sign)
     -> RoundingKind
@@ -323,11 +315,11 @@ pub(crate) fn internal_rounding(kind: RoundingKind, sign: Sign)
             },
         }
     }
-}
+}}
 
-/// Get the global, default rounding scheme.
+// Get the global, default rounding scheme.
+perftools_inline!{
 #[cfg(feature = "correct")]
-#[inline]
 #[allow(unused_variables)]
 pub(crate) fn global_rounding(sign: Sign) -> RoundingKind {
     #[cfg(not(feature = "rounding"))] {
@@ -335,19 +327,16 @@ pub(crate) fn global_rounding(sign: Sign) -> RoundingKind {
     }
 
     #[cfg(feature = "rounding")] {
-        // Use of static mutable `FLOAT_ROUNDING`.
-        unsafe {
-            internal_rounding(FLOAT_ROUNDING, sign)
-        }
+        internal_rounding(get_float_rounding(), sign)
     }
-}
+}}
 
 // TESTS
 // -----
 
 #[cfg(test)]
 mod tests {
-    use float::ExtendedFloat80;
+    use crate::float::ExtendedFloat80;
     use super::*;
 
     // NEAREST ROUNDING
