@@ -1,14 +1,14 @@
 //! Macros for bit-wise shifts.
 
-use lib::mem;
-use util::*;
+use crate::lib::mem;
+use crate::util::*;
 use super::float::ExtendedFloat;
 use super::mantissa::Mantissa;
 
 // SHIFT RIGHT
 
-/// Shift extended-precision float right `shift` bytes.
-#[inline]
+// Shift extended-precision float right `shift` bytes.
+perftools_inline!{
 pub(super) fn shr<M: Mantissa, T: Integer>(fp: &mut ExtendedFloat<M>, shift: T)
 {
     let bits: T = as_cast(mem::size_of::<M>() * 8);
@@ -16,13 +16,13 @@ pub(super) fn shr<M: Mantissa, T: Integer>(fp: &mut ExtendedFloat<M>, shift: T)
 
     fp.mant >>= as_cast::<M, _>(shift);
     fp.exp += shift.as_i32();
-}
+}}
 
-/// Shift extended-precision float right `shift` bytes.
-///
-/// Accepts when the shift is the same as the type size, and
-/// sets the value to 0.
-#[inline]
+// Shift extended-precision float right `shift` bytes.
+//
+// Accepts when the shift is the same as the type size, and
+// sets the value to 0.
+perftools_inline!{
 pub(super) fn overflowing_shr<M: Mantissa, T: Integer>(fp: &mut ExtendedFloat<M>, shift: T)
 {
     let bits: T = as_cast(mem::size_of::<M>() * 8);
@@ -33,10 +33,10 @@ pub(super) fn overflowing_shr<M: Mantissa, T: Integer>(fp: &mut ExtendedFloat<M>
         false => fp.mant >> as_cast::<M, _>(shift),
     };
     fp.exp += shift.as_i32();
-}
+}}
 
-/// Shift extended-precision float left `shift` bytes.
-#[inline]
+// Shift extended-precision float left `shift` bytes.
+perftools_inline!{
 pub(super) fn shl<M: Mantissa, T: Integer>(fp: &mut ExtendedFloat<M>, shift: T)
 {
     let bits: T = as_cast(mem::size_of::<M>() * 8);
@@ -44,4 +44,4 @@ pub(super) fn shl<M: Mantissa, T: Integer>(fp: &mut ExtendedFloat<M>, shift: T)
 
     fp.mant <<= as_cast::<M, _>(shift);
     fp.exp -= shift.as_i32();
-}
+}}
