@@ -3,15 +3,18 @@
 #![allow(unused_imports)]
 
 // Hide implementation details.
-#[macro_use]
-pub(crate) mod api;
-
 #[macro_use]
 mod assert;
 
 #[macro_use]
 mod index;
 
+#[macro_use]
+mod perftools;
+
+#[macro_use]
+mod traits;
+
 #[cfg(test)]
 #[macro_use]
 pub(crate) mod test;
@@ -20,10 +23,13 @@ pub(crate) mod test;
 mod algorithm;
 mod cast;
 mod config;
+mod consume;
+mod div128;
 mod error;
+mod format;
+mod iterator;
 mod mask;
 mod num;
-mod pointer_methods;
 mod primitive;
 mod pow;
 mod result;
@@ -31,12 +37,11 @@ mod rounding;
 mod sign;
 mod table;
 
+#[cfg(feature = "format")]
+mod skip_value;
+
 cfg_if! {
 if #[cfg(feature = "correct")] {
-    mod bound;
-    mod range_bounds;
-    mod slice_index;
-
     #[macro_use]
     mod sequence;
 } else {
@@ -46,15 +51,19 @@ if #[cfg(feature = "correct")] {
 // Publicly export everything with crate-visibility.
 pub(crate) use self::algorithm::*;
 pub(crate) use self::cast::*;
+pub(crate) use self::consume::*;
+pub(crate) use self::div128::*;
+pub(crate) use self::iterator::*;
 pub(crate) use self::mask::*;
-pub(crate) use self::num::*;
-pub(crate) use self::pointer_methods::*;
 pub(crate) use self::primitive::*;
 pub(crate) use self::pow::*;
 pub(crate) use self::rounding::*;
 pub(crate) use self::sign::*;
 pub(crate) use self::table::*;
 
+#[cfg(feature = "format")]
+pub(crate) use self::skip_value::*;
+
 cfg_if! {
 if #[cfg(feature = "correct")] {
     pub(crate) use self::sequence::*;
@@ -64,6 +73,11 @@ if #[cfg(feature = "correct")] {
 
 // Publicly export config globally.
 pub use self::config::*;
-pub use self::error::{Error, ErrorCode, is_empty, is_invalid_digit, is_overflow, is_success};
+pub use self::error::*;
+pub use self::format::*;
+pub use self::num::*;
 pub use self::result::*;
+pub use self::traits::*;
+
+#[cfg(feature = "rounding")]
 pub use self::rounding::RoundingKind;
