@@ -4,14 +4,16 @@
 // Don't delete them, fake they're being used.
 #![allow(dead_code)]
 
-pub(crate) use lib::{f32, f64, mem};
-use lib::{fmt, iter, ops};
+pub(crate) use crate::lib::{f32, f64, mem};
+use crate::lib::{fmt, iter, ops};
 use super::cast::{AsCast, TryCast};
+use super::config::*;
 use super::primitive::Primitive;
 
 // NUMBER
 
 /// Numerical type trait.
+#[doc(hidden)]
 pub trait Number:
     Primitive +
     // Iteration
@@ -27,20 +29,46 @@ pub trait Number:
     ops::RemAssign +
     ops::Sub<Output=Self> +
     ops::SubAssign
-{}
+{
+    /// Maximum number of bytes required to serialize a number to string.
+    const FORMATTED_SIZE: usize;
+    /// Maximum number of bytes required to serialize a number to a decimal string.
+    const FORMATTED_SIZE_DECIMAL: usize;
+    /// If the type can hold a signed (negative) value.
+    const IS_SIGNED: bool;
+}
 
 macro_rules! number_impl {
-    ($($t:ty)*) => ($(
-        impl Number for $t {}
+    ($($t:tt $radix_size:ident $decimal_size:ident $is_signed:expr ; )*) => ($(
+        impl Number for $t {
+            const FORMATTED_SIZE: usize = $radix_size;
+            const FORMATTED_SIZE_DECIMAL: usize = $decimal_size;
+            const IS_SIGNED: bool = $is_signed;
+        }
     )*)
 }
 
-number_impl! { u8 u16 u32 u64 usize i8 i16 i32 i64 isize f32 f64 }
-#[cfg(has_i128)] number_impl! { u128 i128 }
+number_impl! {
+    u8 U8_FORMATTED_SIZE U8_FORMATTED_SIZE_DECIMAL false ;
+    u16 U16_FORMATTED_SIZE U16_FORMATTED_SIZE_DECIMAL false ;
+    u32 U32_FORMATTED_SIZE U32_FORMATTED_SIZE_DECIMAL false ;
+    u64 U64_FORMATTED_SIZE U64_FORMATTED_SIZE_DECIMAL false ;
+    u128 U128_FORMATTED_SIZE U128_FORMATTED_SIZE_DECIMAL false ;
+    usize USIZE_FORMATTED_SIZE USIZE_FORMATTED_SIZE_DECIMAL false ;
+    i8 I8_FORMATTED_SIZE I8_FORMATTED_SIZE_DECIMAL true ;
+    i16 I16_FORMATTED_SIZE I16_FORMATTED_SIZE_DECIMAL true ;
+    i32 I32_FORMATTED_SIZE I32_FORMATTED_SIZE_DECIMAL true ;
+    i64 I64_FORMATTED_SIZE I64_FORMATTED_SIZE_DECIMAL true ;
+    i128 I128_FORMATTED_SIZE I128_FORMATTED_SIZE_DECIMAL true ;
+    isize ISIZE_FORMATTED_SIZE ISIZE_FORMATTED_SIZE_DECIMAL true ;
+    f32 F32_FORMATTED_SIZE F32_FORMATTED_SIZE_DECIMAL true ;
+    f64 F64_FORMATTED_SIZE F64_FORMATTED_SIZE_DECIMAL true ;
+}
 
 // INTEGER
 
 /// Defines a trait that supports integral operations.
+#[doc(hidden)]
 pub trait Integer:
     // Basic
     Number + Eq + Ord +
@@ -231,7 +259,6 @@ pub trait Integer:
         try_cast_or_max(self)
     }
 
-    #[cfg(has_i128)]
     #[inline]
     fn try_u128_or_max(self) -> u128 {
         try_cast_or_max(self)
@@ -262,7 +289,6 @@ pub trait Integer:
         try_cast_or_max(self)
     }
 
-    #[cfg(has_i128)]
     #[inline]
     fn try_i128_or_max(self) -> i128 {
         try_cast_or_max(self)
@@ -295,7 +321,6 @@ pub trait Integer:
         try_cast_or_min(self)
     }
 
-    #[cfg(has_i128)]
     #[inline]
     fn try_u128_or_min(self) -> u128 {
         try_cast_or_min(self)
@@ -326,7 +351,6 @@ pub trait Integer:
         try_cast_or_min(self)
     }
 
-    #[cfg(has_i128)]
     #[inline]
     fn try_i128_or_min(self) -> i128 {
         try_cast_or_min(self)
@@ -521,36 +545,36 @@ macro_rules! integer_impl {
     )*)
 }
 
-integer_impl! { u8 u16 u32 u64 usize i8 i16 i32 i64 isize }
-#[cfg(has_i128)] integer_impl! { u128 i128 }
+integer_impl! { u8 u16 u32 u64 u128 usize i8 i16 i32 i64 i128 isize }
 
 /// Unwrap or get T::max_value().
 #[inline]
-pub fn unwrap_or_max<T: Integer>(t: Option<T>) -> T {
+pub(crate) fn unwrap_or_max<T: Integer>(t: Option<T>) -> T {
     t.unwrap_or(T::max_value())
 }
 
 /// Unwrap or get T::min_value().
 #[inline]
-pub fn unwrap_or_min<T: Integer>(t: Option<T>) -> T {
+pub(crate) fn unwrap_or_min<T: Integer>(t: Option<T>) -> T {
     t.unwrap_or(T::min_value())
 }
 
 /// Try to convert to U, if not, return U::max_value().
 #[inline]
-pub fn try_cast_or_max<U: Integer, T: TryCast<U>>(t: T) -> U {
+pub(crate) fn try_cast_or_max<U: Integer, T: TryCast<U>>(t: T) -> U {
     unwrap_or_max(TryCast::try_cast(t))
 }
 
 /// Try to convert to U, if not, return U::min_value().
 #[inline]
-pub fn try_cast_or_min<U: Integer, T: TryCast<U>>(t: T) -> U {
+pub(crate) fn try_cast_or_min<U: Integer, T: TryCast<U>>(t: T) -> U {
     unwrap_or_min(TryCast::try_cast(t))
 }
 
 // SIGNED INTEGER
 
 /// Defines a trait that supports signed integral operations.
+#[doc(hidden)]
 pub trait SignedInteger: Integer + ops::Neg<Output=Self>
 {
     // FUNCTIONS (INHERITED)
@@ -577,12 +601,12 @@ macro_rules! signed_integer_impl {
     )*)
 }
 
-signed_integer_impl! { i8 i16 i32 i64 isize }
-#[cfg(has_i128)] signed_integer_impl! { i128 }
+signed_integer_impl! { i8 i16 i32 i64 i128 isize }
 
 // UNSIGNED INTEGER
 
 /// Defines a trait that supports unsigned integral operations.
+#[doc(hidden)]
 pub trait UnsignedInteger: Integer
 {
     /// Returns true if the least-significant bit is odd.
@@ -604,12 +628,12 @@ macro_rules! unsigned_integer_impl {
     )*)
 }
 
-unsigned_integer_impl! { u8 u16 u32 u64 usize }
-#[cfg(has_i128)] unsigned_integer_impl! { u128 }
+unsigned_integer_impl! { u8 u16 u32 u64 u128 usize }
 
 // FLOAT
 
 /// Float information for native float types.
+#[doc(hidden)]
 pub trait Float: Number + ops::Neg<Output=Self>
 {
     /// Unsigned type of the same size.
@@ -817,11 +841,14 @@ pub trait Float: Number + ops::Neg<Output=Self>
 
 /// Wrap float method for `std` and `no_std` context.
 macro_rules! float_method {
-    ($f:ident, $t:tt, $meth:ident, $intr:ident $(,$i:expr)*) => ({
+    ($f:ident, $t:tt, $meth:ident, $intr:ident, $libm:ident $(,$i:expr)*) => ({
         #[cfg(feature = "std")]
         return $t::$meth($f $(,$i)*);
 
-        #[cfg(not(feature = "std"))]
+        #[cfg(all(not(feature = "std"), feature = "libm"))]
+        return libm::$libm($f $(,$i)*);
+
+        #[cfg(all(not(feature = "std"), not(feature = "libm")))]
         return unsafe { core::intrinsics::$intr($f $(,$i)*) };
     })
 }
@@ -831,14 +858,17 @@ macro_rules! float_method {
 /// This is because MSVC wraps these as inline functions, with no actual
 /// ABI for the LLVM intrinsic.
 macro_rules! float_method_msvc {
-    ($f:ident, $ts:tt, $tl:tt, $meth:ident, $intr:ident $(,$i:expr)*) => ({
+    ($f:ident, $ts:tt, $tl:tt, $meth:ident, $intr:ident, $libm:ident $(,$i:expr)*) => ({
         #[cfg(feature = "std")]
         return $ts::$meth($f $(,$i)*);
 
-        #[cfg(all(not(feature = "std"), not(target_env = "msvc")))]
+        #[cfg(all(not(feature = "std"), feature = "libm"))]
+        return libm::$libm($f $(,$i)*);
+
+        #[cfg(all(not(feature = "std"), not(feature = "libm"), not(target_env = "msvc")))]
         return unsafe { core::intrinsics::$intr($f $(,$i)*) };
 
-        #[cfg(all(not(feature = "std"), target_env = "msvc"))]
+        #[cfg(all(not(feature = "std"), not(feature = "libm"), target_env = "msvc"))]
         return $tl::$meth($f as $tl $(,$i as $tl)*) as $ts;
     })
 }
@@ -848,15 +878,18 @@ macro_rules! float_method_msvc {
 /// Solaris has a standard non-conforming log implementation, we need
 /// to wrap this cheaply.
 macro_rules! float_method_log_solaris {
-    ($f:ident, $t:tt, $meth:ident, $intr:ident $(,$i:expr)*) => ({
+    ($f:ident, $t:tt, $meth:ident, $intr:ident, $libm:ident $(,$i:expr)*) => ({
         #[cfg(feature = "std")]
         return $t::$meth($f $(,$i)*);
 
-        #[cfg(all(not(feature = "std"), not(target_os = "solaris")))]
+        #[cfg(all(not(feature = "std"), feature = "libm"))]
+        return libm::$libm($f $(,$i)*);
+
+        #[cfg(all(not(feature = "std"), not(feature = "libm"), not(target_os = "solaris")))]
         return unsafe { core::intrinsics::$intr($f $(,$i)*) };
 
         // Workaround for Solaris/Illumos due to log(-value) == -Inf, not NaN.
-        #[cfg(all(not(feature = "std"), target_os = "solaris"))] {
+        #[cfg(all(not(feature = "std"), not(feature = "libm"), target_os = "solaris"))] {
             if $f.is_nan() {
                 $f
             } else if $f.is_special() {
@@ -896,42 +929,48 @@ impl Float for f32 {
 
     #[inline]
     fn abs(self) -> f32 {
-        float_method!(self, f32, abs, fabsf32)
+        float_method!(self, f32, abs, fabsf32, fabsf)
     }
 
     #[inline]
     fn ceil(self) -> f32 {
-        float_method_msvc!(self, f32, f64, ceil, ceilf32)
+        float_method_msvc!(self, f32, f64, ceil, ceilf32, ceilf)
     }
 
     #[inline]
     fn exp(self) -> f32 {
-        float_method_msvc!(self, f32, f64, exp, expf32)
+        float_method_msvc!(self, f32, f64, exp, expf32, expf)
     }
 
     #[inline]
     fn floor(self) -> f32 {
-        float_method_msvc!(self, f32, f64, floor, floorf32)
+        float_method_msvc!(self, f32, f64, floor, floorf32, floorf)
     }
 
     #[inline]
     fn ln(self) -> f32 {
-        float_method_msvc!(self, f32, f64, ln, logf32)
+        float_method_msvc!(self, f32, f64, ln, logf32, logf)
     }
 
     #[inline]
     fn powi(self, n: i32) -> f32 {
-        float_method!(self, f32, powi, powif32, n)
+        cfg_if! {
+            if #[cfg(all(not(feature = "std"), feature = "libm"))] {
+                self.powf(n as f32)
+            } else {
+                float_method!(self, f32, powi, powif32, powif, n)
+            }
+        }
     }
 
     #[inline]
     fn powf(self, n: f32) -> f32 {
-        float_method_msvc!(self, f32, f64, powf, powf32, n)
+        float_method_msvc!(self, f32, f64, powf, powf32, powf, n)
     }
 
     #[inline]
     fn round(self) -> f32 {
-        float_method!(self, f32, round, roundf32)
+        float_method!(self, f32, round, roundf32, roundf)
     }
 
     #[inline]
@@ -979,42 +1018,48 @@ impl Float for f64 {
 
     #[inline]
     fn abs(self) -> f64 {
-        float_method!(self, f64, abs, fabsf64)
+        float_method!(self, f64, abs, fabsf64, fabs)
     }
 
     #[inline]
     fn ceil(self) -> f64 {
-        float_method!(self, f64, ceil, ceilf64)
+        float_method!(self, f64, ceil, ceilf64, ceil)
     }
 
     #[inline]
     fn exp(self) -> f64 {
-        float_method!(self, f64, exp, expf64)
+        float_method!(self, f64, exp, expf64, exp)
     }
 
     #[inline]
     fn floor(self) -> f64 {
-        float_method!(self, f64, floor, floorf64)
+        float_method!(self, f64, floor, floorf64, floor)
     }
 
     #[inline]
     fn ln(self) -> f64 {
-        float_method_log_solaris!(self, f64, ln, logf64)
+        float_method_log_solaris!(self, f64, ln, logf64, log)
     }
 
     #[inline]
     fn powi(self, n: i32) -> f64 {
-        float_method!(self, f64, powi, powif64, n)
+        cfg_if! {
+            if #[cfg(all(not(feature = "std"), feature = "libm"))] {
+                self.powf(n as f64)
+            } else {
+                float_method!(self, f64, powi, powif64, powi, n)
+            }
+        }
     }
 
     #[inline]
     fn powf(self, n: f64) -> f64 {
-        float_method!(self, f64, powf, powf64, n)
+        float_method!(self, f64, powf, powf64, pow, n)
     }
 
     #[inline]
     fn round(self) -> f64 {
-        float_method!(self, f64, round, roundf64)
+        float_method!(self, f64, round, roundf64, round)
     }
 
     #[inline]
